@@ -2,6 +2,7 @@
 require 'test/unit'
 require 'tempfile'
 require "thread"
+require "-test-/file"
 require_relative 'ut_eof'
 
 class TestFile < Test::Unit::TestCase
@@ -121,8 +122,8 @@ class TestFile < Test::Unit::TestCase
 
   def test_truncate_size
     Tempfile.create("test-truncate") do |f|
-      q1 = Queue.new
-      q2 = Queue.new
+      q1 = Thread::Queue.new
+      q2 = Thread::Queue.new
 
       th = Thread.new do
         data = ''
@@ -268,7 +269,11 @@ class TestFile < Test::Unit::TestCase
       realdir = File.realpath(tmpdir)
       open(File.join(tmpdir, tst), "w") {}
       a = File.join(tmpdir, "x")
-      File.symlink(tst, a)
+      begin
+        File.symlink(tst, a)
+      rescue Errno::EACCES, Errno::EPERM
+        skip "need privilege"
+      end
       assert_equal(File.join(realdir, tst), File.realpath(a))
       File.unlink(a)
 
@@ -279,6 +284,14 @@ class TestFile < Test::Unit::TestCase
     }
   end
 
+  def test_realpath_special_symlink
+    IO.pipe do |r, w|
+      if File.pipe?(path = "/dev/fd/#{r.fileno}")
+        assert_file.identical?(File.realpath(path), path)
+      end
+    end
+  end
+
   def test_realdirpath
     Dir.mktmpdir('rubytest-realdirpath') {|tmpdir|
       realdir = File.realpath(tmpdir)
@@ -354,6 +367,7 @@ class TestFile < Test::Unit::TestCase
       sleep 2
       File.write(path, "bar")
       sleep 2
+      File.read(path)
       File.chmod(0644, path)
       sleep 2
       File.read(path)
@@ -365,7 +379,7 @@ class TestFile < Test::Unit::TestCase
       if stat.birthtime != stat.ctime
         assert_in_delta t0+4, stat.ctime.to_f, delta
       end
-      unless /mswin|mingw/ =~ RUBY_PLATFORM
+      if /mswin|mingw/ !~ RUBY_PLATFORM && !Bug::File::Fs.noatime?(path)
         # Windows delays updating atime
         assert_in_delta t0+6, stat.atime.to_f, delta
       end
