@@ -141,7 +141,9 @@ class TestGemPackage < Gem::Package::TarTestCase
 
     FileUtils.mkdir_p 'lib'
     open 'lib/code.rb',  'w' do |io| io.write '# lib/code.rb'  end
-    File.symlink('lib/code.rb', 'lib/code_sym.rb')
+
+    # NOTE: 'code.rb' is correct, because it's relative to lib/code_sym.rb
+    File.symlink('code.rb', 'lib/code_sym.rb')
 
     package = Gem::Package.new 'bogus.gem'
     package.spec = spec
@@ -156,12 +158,16 @@ class TestGemPackage < Gem::Package::TarTestCase
 
     Gem::Package::TarReader.new tar do |tar_io|
       tar_io.each_entry do |entry|
-        (entry.symlink? ? symlinks : files) << entry.full_name
+        if entry.symlink?
+          symlinks << { entry.full_name => entry.header.linkname }
+        else
+          files << entry.full_name
+        end
       end
     end
 
     assert_equal %w[lib/code.rb], files
-    assert_equal %w[lib/code_sym.rb], symlinks
+    assert_equal [{'lib/code_sym.rb' => 'lib/code.rb'}], symlinks
   end
 
   def test_build
